/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/
 
// Implementation of class Cmp3PlayList

#include "mp3_playlist.h"
#include "mp3_source_config.h"
#include <id3/tag.h>
#include <zz_file.h>
#include <zz_path.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 

// Construction
Cmp3PlayList::Cmp3PlayList( const CzzString& usr )
 : m_nowPlayingFileHande(NULL), m_nowPlayingSong( Cmp3SourceConfig::getPauseMessage() ), m_nowPlaying(0), m_skipIndicator(0), m_Log( Cmp3SourceConfig::appName().c_str() ), m_User(usr), m_BitRate(128)
{
	readRadioPlayList();
}

// Destruction
Cmp3PlayList::~Cmp3PlayList()
{
	if( m_nowPlayingFileHande )
		fclose( m_nowPlayingFileHande );

	saveRadioPlayList();
}

void Cmp3PlayList::readRadioPlayList()
{
	CzzString pl = CzzString::Format( "%s\\My radio - %s.m3u", Cmp3SourceConfig::getM3uDir().c_str(), m_User.c_str() );

	char file[2048];
	FILE* f = fopen( pl.c_str(), "r" );
	int p = 0;
	int idx = -1;
	CzzString localpath;

	if( f != NULL )
	{
		while( fgets( file, 2048, f ) != NULL )
		{
			if( file[0] != '#' )
			{
				file[strlen(file)-1]='\0';
				
				p = 0;
				for( int i = strlen(file); i >= 0; i-- )
				{
					if( file[i] == '\\' )
					{
						p++;

						if( p == 3 )
						{
							idx++;
							localpath = file+i+1;
							m_StringMap[idx] = localpath;
							m_IndexMap[localpath] = idx;
						}
					}
				}
			}
		}

		fclose( f ) ;
	}
}

void Cmp3PlayList::saveRadioPlayList()
{
	CzzString pl = CzzString::Format( "%s\\My radio - %s.m3u", Cmp3SourceConfig::getM3uDir().c_str(), m_User.c_str() );
	CzzString dir = Cmp3SourceConfig::getDataDir();

	FILE* f = fopen( pl.c_str(), "w" );

	for( int i=0; i < m_StringMap.size(); i++ )
		fprintf( f, "%s\\%s\n", dir.c_str(), m_StringMap[i].c_str() ); 

	fclose(f);
}

void Cmp3PlayList::add( const char* name, const char* album, const char* song )
{
	CzzString entry = CzzString::Format( "%s\\%s\\%s", name, album, song );
	CzzString filename = CzzString::Format( "%s\\%s", Cmp3SourceConfig::getDataDir().c_str(), entry.c_str() );
	
	m_Log.Print( _DEBUGPARAMS, "Trying to add %s", filename.c_str() );

	CzzFile file( filename.c_str() );

	m_Mutex.Acquire( true );

	if( file.Exist() )
	{
		unsigned int i = m_StringMap.size();

		// Kolla om sngen redan finns i listan
		if( m_IndexMap.find( entry ) == m_IndexMap.end() )
		{
			m_StringMap[i] = entry;
			m_IndexMap[entry] = i;
			m_Log.Print( _DEBUGPARAMS, "Added song '%s', now %d in queue", entry.c_str(), m_StringMap.size() );
		}
		
		m_Log.Print( _DEBUGPARAMS, "Song '%s' already in list, now %d in queue", entry.c_str(), m_StringMap.size() );
	}

	m_Mutex.Release();
}

void Cmp3PlayList::forward()
{
	m_Mutex.Acquire( true );

	m_skipIndicator++;

	if( m_skipIndicator + m_nowPlaying > m_StringMap.size() )
		m_skipIndicator = 0;

	m_Mutex.Release();
}

void Cmp3PlayList::randomize()
{
	m_Mutex.Acquire( true );

	srand( (unsigned)time( NULL ) );
	int i = 0; 
	int rno = 0;
	float k = (float)((float)RAND_MAX/(float)m_StringMap.size());
	map<CzzString, unsigned int>::iterator iter;

	iter = m_IndexMap.begin();
	m_StringMap.clear();

	// Indexera om
	while( m_StringMap.size() < m_IndexMap.size() )
	{
		rno = (rand()/k);

		if( m_StringMap.find( rno ) == m_StringMap.end() )
		{
			iter->second = rno;
			m_StringMap[rno] = iter->first;
			iter++;
		}
	}

	m_nowPlaying = 0;
	m_skipIndicator = 0;

	m_Mutex.Release();
}

void Cmp3PlayList::reverse()
{
	m_Mutex.Acquire( true );

	m_skipIndicator--;

	if( m_skipIndicator + m_nowPlaying < 0 )
		m_skipIndicator = m_StringMap.size() - m_nowPlaying - 1;

	m_Mutex.Release();
}

void Cmp3PlayList::replacePlaylist( const char* name )
{
	m_Mutex.Acquire( true );

	CzzString from = CzzString::Format( "%s\\%s", Cmp3SourceConfig::getM3uDir().c_str(), name );
	CzzString to = CzzString::Format( "%s\\My radio - %s.m3u", Cmp3SourceConfig::getM3uDir().c_str(), m_User.c_str() );

	m_Log.Print( _DEBUGPARAMS, "Trying to replace playlist %s -> %s", from.c_str(), to.c_str() );

	CzzFile pl( from.c_str() );

	if( pl.Exist() )
	{
		pl.CopyTo( to.c_str() );
		pl.Close();

		m_StringMap.clear();
		m_IndexMap.clear();

		m_nowPlaying = 0;
		m_skipIndicator = 0;
		readRadioPlayList();
	}

	m_Mutex.Release();
}

void Cmp3PlayList::remove( const char* name, const char* album, const char* song )
{
	CzzString entry = CzzString::Format( "%s\\%s\\%s", name, album, song );
	unsigned int index = 0;

	m_Log.Print( _DEBUGPARAMS, "Trying to remove %s", entry.c_str() );

	m_Mutex.Acquire( true );

	if( m_IndexMap.find( entry ) != m_IndexMap.end() )
	{
		index = m_IndexMap[entry];
		m_StringMap.erase( index );
		m_IndexMap.erase( entry );

		m_Log.Print( _DEBUGPARAMS, "Removed song '%s', now %d in queue", entry.c_str(), m_StringMap.size() );

		m_StringMap.clear();

		if( m_IndexMap.size() == 0 )
		{
			m_nowPlaying = 0;
			m_nowPlayingFileHande = 0;
		}
		else
		{
			map<CzzString, unsigned int>::iterator iter;

			index = 0;

			// Indexera om
			for( iter = m_IndexMap.begin(); iter != m_IndexMap.end(); iter++ )
			{
				if( iter->second != index )	
					iter->second = index;

				m_StringMap[index] = iter->first;

				index++;
			}
		}
	}
	
	m_Mutex.Release();
}

void Cmp3PlayList::clear()
{
	m_Mutex.Acquire( true );

	m_StringMap.clear();
	m_IndexMap.clear();
	m_nowPlayingFileHande = NULL;
	m_nowPlaying = 0;
	
	m_Log.Print( _DEBUGPARAMS, "Clear playlist now %d in queue", m_StringMap.size() );

	m_Mutex.Release();
}

FILE* Cmp3PlayList::getNextStreamHandle()
{
	m_Mutex.Acquire( true );

	if( m_StringMap.size() > 0 )
	{
		if( m_nowPlayingFileHande )
			fclose( m_nowPlayingFileHande );

		m_nowPlaying = m_nowPlaying + 1 + m_skipIndicator;
		m_skipIndicator = 0;

		if( m_nowPlaying > m_StringMap.size() )
			m_nowPlaying = 1;
		
		if( m_nowPlaying < 1 )
			m_nowPlaying = m_StringMap.size();

		CzzString file = CzzString::Format( "%s\\%s", Cmp3SourceConfig::getDataDir().c_str(), m_StringMap[m_nowPlaying-1].c_str() );
		m_BitRate = getBitRate( file.c_str() );
		m_Log.Print( _INFOPARAMS, "Playing %d:%s at %d kbps", m_nowPlaying, file.c_str(), m_BitRate );
		m_nowPlayingFileHande = fopen( file.c_str(), "rb" );
	}
	else
	{
		if( m_nowPlayingFileHande )
			fclose( m_nowPlayingFileHande );

		CzzString file = Cmp3SourceConfig::getPauseSong();
		m_BitRate = getBitRate( file.c_str() );
		m_Log.Print( _INFOPARAMS, "Playing %d:%s at %d kbps", m_nowPlaying, file.c_str(), m_BitRate );
		m_nowPlayingFileHande = fopen( file.c_str(), "rb" );
	}

	if( m_nowPlaying > 0 )
	{
		CzzString file = CzzString::Format( "%s\\%s", Cmp3SourceConfig::getDataDir().c_str(), m_StringMap[m_nowPlaying-1].c_str() );
		m_nowPlayingSong = Cmp3PlayList::readID3( file.c_str() );
	}
	else
		m_nowPlayingSong = Cmp3SourceConfig::getPauseMessage();

	m_Mutex.Release();	

	return m_nowPlayingFileHande;
}

CzzString Cmp3PlayList::list()
{
	CzzString result;
	CzzString file;

	map<unsigned int, CzzString>::iterator i;

	for( i = m_StringMap.begin(); i != m_StringMap.end(); i++ )
	{
		file = CzzString::Format( "%s\\%s", Cmp3SourceConfig::getDataDir().c_str(), m_StringMap[ i->first ].c_str() );
		result = result + Cmp3PlayList::readID3( file.c_str() ) + "\n";
		result = result + m_StringMap[ i->first ] + "\n";
	}

	return result;
}

CzzString Cmp3PlayList::listIndexes()
{
	CzzString result;
	CzzString file;

	map<unsigned int, CzzString>::iterator i;

	for( i = m_StringMap.begin(); i != m_StringMap.end(); i++ )
		result = result + m_StringMap[ i->first ] + "\n";

	return result;
}

CzzString Cmp3PlayList::readID3( const char* file )
{
	CzzString result;
	ID3_Frame* frame = NULL;
	ID3_Field* fld = NULL;
	ID3_Tag tag( file );
	const Mp3_Headerinfo* hdr = tag.GetMp3HeaderInfo();
	CzzString title;
	CzzString artist;
	uint32 minutes = 0;
	uint32 secs = 0;
	uint32 speed = 128;

	if( hdr != NULL )
	{
		minutes = hdr->time/60;
		secs = hdr->time - (minutes*60);
		if( hdr->bitrate != 0 )
			speed = hdr->bitrate/1000;
		if( hdr->vbr_bitrate != 0 )
			speed = hdr->vbr_bitrate/1000;
	}

	frame = tag.Find( ID3FID_TITLE );

	if( frame != NULL )
	{
		fld = frame->GetField( ID3FN_TEXT );

		if( fld != NULL )
		{
            fld->SetEncoding(ID3TE_ISO8859_1);
			title = fld->GetRawText();

            frame = tag.Find( ID3FID_LEADARTIST );

	        if( frame != NULL )
	        {
                fld->SetEncoding(ID3TE_ISO8859_1);
		        fld = frame->GetField( ID3FN_TEXT );

		        if( fld != NULL )
		        {
                    fld->SetEncoding(ID3TE_ISO8859_1);
			        artist = fld->GetRawText();
		        }
	        }
		}
        else
            title = CzzPath::strip( file );
	}
    else
        title = CzzPath::strip( file );
	
	if( artist.size() )
		result = CzzString::Format( "%s - %s", artist.c_str(), title.c_str() );
	else
		result = CzzString::Format( "%s", title.c_str() );;

	return result;
}

CzzString Cmp3PlayList::nowPlaying()
{
	m_Mutex.Acquire( true );

	CzzString r =  m_nowPlayingSong;

	m_Mutex.Release();

	return r;
}

CzzString Cmp3PlayList::commingUp()
{
	m_Mutex.Acquire( true );

	int i = m_nowPlaying + m_skipIndicator;
	CzzString result;

	if( m_StringMap.size() )
	{
		i++;

		if( i > m_StringMap.size() )
			i = 1;

		if( i < 1 )
			i = m_StringMap.size();

		result = Cmp3PlayList::readID3( CzzString::Format( "%s\\%s", Cmp3SourceConfig::getDataDir().c_str(), m_StringMap[i-1].c_str() ).c_str() );
	}
	else
		result = Cmp3SourceConfig::getPauseMessage();

	m_Mutex.Release();

	return result;
}

CzzString Cmp3PlayList::lastPlayed()
{
	m_Mutex.Acquire( true );

	int i = m_nowPlaying + m_skipIndicator;
	CzzString result;

	if( m_StringMap.size() )
	{
		i--;

		if( i <= 0 )
			i = 1;

		result = Cmp3PlayList::readID3( CzzString::Format( "%s\\%s", Cmp3SourceConfig::getDataDir().c_str(), m_StringMap[i-1].c_str() ).c_str() );
	}
	else
		result = Cmp3SourceConfig::getPauseMessage();

	m_Mutex.Release();

	return result;
}

int Cmp3PlayList::getBitRate( void )
{
	return m_BitRate;
}

int Cmp3PlayList::getBitRate( const char* file )
{
	uint32 speed = 128;
	ID3_Tag tag( file );
	const Mp3_Headerinfo* hdr = tag.GetMp3HeaderInfo();
	
	if( hdr != NULL )
	{
		if( hdr->bitrate != 0 )
			speed = hdr->bitrate/1000;
		if( hdr->vbr_bitrate != 0 )
			speed = hdr->vbr_bitrate/1000;
	}

	return speed;
}