/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3LatestListData

#include "mp3_latest_list_data.h"
#include "mp3_svc.h"
#include "mp3_config.h"

#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h> 
#include <id3/tag.h>
#include <zz_file.h>
#include <winver.h>

// Construction
Cmp3LatestListData::Cmp3LatestListData( void )
: CzzDataService( "listLatestData" )
{

}

// Destruction
Cmp3LatestListData::~Cmp3LatestListData()
{

}

CzzDataServiceReply* Cmp3LatestListData::execute( void* session, CzzParameterMap* req )
{
	CzzDataService::execute( session, req );

	CzzDataServiceReply* reply = new CzzDataServiceReply();

	reply->addFieldName( MP3_FLD_LIST_ART );
	reply->addFieldName( MP3_FLD_TITLE_ROW );
	reply->addFieldName( MP3_FLD_INFO_ROW );
	reply->addFieldName( MP3_FLD_TITLE_LINK );
	reply->addFieldName( MP3_FLD_PLAY_LINK );


	CzzString path = Cmp3Config::getDataDir();
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.*").c_str(), &finfo );
	int error = 0;
	int albums = 0;
	int artists = 0;
	int songs = 0;
	Cmp3List latest_added;

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
				{
					countAlbumsAndSongs( finfo.name, (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), &albums, &songs, &latest_added );
					artists++;
				}

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );

	
	struct tm* pt;
	time_t t;
	Cmp3ListItem* obj = NULL;
	int i = 1;
	CzzString added;

	while( ( obj = latest_added.next()) != latest_added.last() && i <= Cmp3Config::getMaxResultSize() )
	{
		t = obj->getTime();
		pt = localtime( &t );

		/*
		reply->addFieldName( MP3_FLD_LIST_ART );
		reply->addFieldName( MP3_FLD_TITLE_ROW );
		reply->addFieldName( MP3_FLD_INFO_ROW );
		reply->addFieldName( MP3_FLD_TITLE_LINK );
		reply->addFieldName( MP3_FLD_PLAY_LINK );
		*/

		added = CzzString::Format( "[%4d-%02d-%02d %02d.%02d]<br>\n", pt->tm_year+1900, pt->tm_mon+1, pt->tm_mday, 
				pt->tm_hour, pt->tm_min );

		reply->addFieldValue( urlEncode( CzzString::Format( "%s/%s/%s/Folder.jpg", Cmp3Config::getMusicRoot().c_str(), urlEncode(obj->getName()).c_str(), urlEncode(obj->getAlbum()).c_str() ) ) );
		reply->addFieldValue( CzzString::Format( "%s - %s", obj->getName().c_str(), obj->getAlbum().c_str() ) );
		reply->addFieldValue( added );
		reply->addFieldValue( CzzString::Format( "%s/mp3_gui.exe?svc=%d&name=%s&album=%s",
			Cmp3Config::getCGIDir().c_str(), MP3_SVC_LIST_SONGS, urlEncode( obj->getName() ).c_str(), urlEncode(obj->getAlbum()).c_str() ) );
		reply->addFieldValue( CzzString::Format( "%s/mp3_playlist.exe?svc=%d&name=%s&album=%s",
			Cmp3Config::getCGIDir().c_str(), 1, urlEncode(obj->getName()).c_str(), urlEncode(obj->getAlbum()).c_str() ) );

		i++;
	}

	return reply;
}

void Cmp3LatestListData::countAlbumsAndSongs( const char* artist, const char* path, int* albums, int* songs, Cmp3List* list )
{
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.*").c_str(), &finfo );
	int error = 0;
	CzzString cgidir = Cmp3Config::getCGIDir();
	CzzString url;

	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
				{
					countSongs( (CzzString(path) + "\\" + CzzString(finfo.name)).c_str(), songs, list );
					(*albums)++;
					list->add( artist, finfo.name, finfo.time_write );
				}

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

void Cmp3LatestListData::countSongs( const char* path, int* songs, Cmp3List* list )
{
	_finddata_t finfo;
	long handle = _findfirst( (CzzString(path) + "\\*.mp3").c_str(), &finfo );
	int error = 0;
	
	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				if( !(finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
					(*songs)++;

			}
			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

int Cmp3LatestListData::size()
{
    return sizeof(Cmp3LatestListData);
}