/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3List

#include <limits.h> 
#include "mp3_list.h"
#include <algorithm>

// Construction
Cmp3List::Cmp3List( void ) : m_List(), m_Sorted(false), m_DoSort(true), m_Init(false)
{

}

Cmp3List::Cmp3List( bool sorted ) : m_List(), m_Sorted(false), m_DoSort(sorted), m_Init(false)
{

}

// Destruction
Cmp3List::~Cmp3List()
{
	vector<Cmp3ListItem*>::iterator i;
	
	// Free the objects pointed to in the vector
	for( i = m_List.begin(); i != m_List.end(); i++ )
		delete *i;

	m_List.clear();
}

int Cmp3List::size( void )
{
	return m_List.size();
}

void Cmp3List::add( const char* item, long time )
{
	m_Sorted = false;

	Cmp3ListItem* obj = new Cmp3ListItem( time, item );
	m_List.push_back(obj);
}

void Cmp3List::add( const char* name, const char* album, long time )
{
	m_Sorted = false;

	Cmp3ListItem* obj = new Cmp3ListItem( time, name, album );
	m_List.push_back(obj);
}

void Cmp3List::sortList( void )
{
	if( m_DoSort )
		sort( m_List.begin(), m_List.end(), Cmp3ListItem() );
}

// Get first service in the map
Cmp3ListItem* Cmp3List::first( void )
{
	if( !m_Sorted )
		sortList();

	return *m_List.begin();
}

// Get Last service in the map
Cmp3ListItem* Cmp3List::last( void )
{
	if( !m_Sorted )
		sortList();

	return *m_List.end();
}

// Get next service in the map
Cmp3ListItem* Cmp3List::next( void )
{
	if( !m_Sorted )
		sortList();

	if( !m_Init )
	{
		m_Iterator = m_List.begin();
		m_Init = true;
		return *m_Iterator;
	}

	if( m_Iterator != m_List.end() )
		m_Iterator++;
	else
		m_Iterator = m_List.end();
	
	return *m_Iterator;
}

// Rewind iterator
void Cmp3List::rewind( void )
{
	m_Init = false;
}


