/* myMusic
 *
 * This program is distributed under the GNU General Public License, version 2.
 * A copy of this license is included with this source.
 *
 * Copyright 2004-2006, Toni Thomsson <toni@tonjac.org> 
*/

// Implementation of class Cmp3M3UPlayList

#include "mp3_m3u_playlist.h"
#include <mp3_config.h>
#include <zz_file.h>

// Construction
Cmp3M3UPlayList::Cmp3M3UPlayList( const CzzString& name )
{
    long handle = 0;
	int error = 0;
	_finddata_t finfo;
	Cmp3Config cfg;

	handle = _findfirst( ( cfg.getDataDir() + "\\" + name + "\\*.*").c_str(), &finfo );
	
	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				// Is it a subdir?
				if( (finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
					listSongsForAlbum( false, name, CzzString(finfo.name), CzzString( cfg.getDataDir() + "\\" + name + "\\" + finfo.name + "\\*.mp3") );
			}

			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

void Cmp3M3UPlayList::listSongsForAlbum( bool orderd, const CzzString& name, const CzzString& album, const CzzString& path )
{
	long handle = 0;
	int error = 0;
	_finddata_t finfo;
	Cmp3Config cfg;
	handle = _findfirst( path.c_str(), &finfo );
	
	// Read the dir
	if( (handle!= ENOENT)  && (handle != EINVAL) && (handle != -1) )
	{
		while( !error )
		{
			// We're not interested in the . and .. 
			if( (stricmp( finfo.name, "." ) != 0) && (stricmp( finfo.name, ".." ) != 0) )
			{
				// Is it a subdir?
				if( !(finfo.attrib & _A_SUBDIR) && !(finfo.attrib & _A_SYSTEM) )
				{
					Cmp3Track* track = new Cmp3Track( name, album, ( cfg.getDataDir() + "\\" + name + "\\" + album + "\\" + finfo.name ).c_str() );

                    int no = 0;

                    if( orderd )
                    {
                        no = track->getNumber();

                        if( no == 0 || m_List.find( no ) != m_List.end() )
                            no = m_List.size()+1;
                    }
                    else
                        no = m_List.size()+1;

                    m_List[ no ] = track;
				}
			}

			// Next file
			error = _findnext( handle, &finfo );
		}
	}

	_findclose( handle );
}

Cmp3M3UPlayList::Cmp3M3UPlayList( const CzzString& name, const CzzString& album )
{
    Cmp3Config cfg;
    listSongsForAlbum( true, name, album, CzzString( cfg.getDataDir() + "\\" + name + "\\" + album + "\\*.mp3") );
}

Cmp3M3UPlayList::Cmp3M3UPlayList( const CzzString& name, const CzzString& album, const CzzString& song )
{
    Cmp3Config cfg;
    Cmp3Track* track = new Cmp3Track( name, album, ( cfg.getDataDir() + "\\" + name + "\\" + album + "\\" + song ).c_str() );
    m_List[ 1 ] = track;
}

// Destruction
Cmp3M3UPlayList::~Cmp3M3UPlayList()
{
    map<int, Cmp3Track* >::iterator i;

    for( i = m_List.begin(); i != m_List.end(); i++ )
		delete i->second;
}

CzzString Cmp3M3UPlayList::toString( void )
{
    CzzString result;
    Cmp3Config cfg;

    result = result + CzzString( "#EXTM3U\n" );
    
    for( int i = 1; i <= m_List.size(); i++ )
    {
        Cmp3Track* track = m_List[i];
        CzzString url = CzzString::Format( "%s/%s/%s/%s", (cfg.getWWWHost() +  "/mp3").c_str(), track->getArtist().c_str(), track->getAlbum().c_str(), track->getFile().c_str() );
        result = result + CzzString::Format( "#EXTINF:%d,%s - %s\n", track->getTime(), track->getArtist().c_str(), track->getTitle().c_str() );
        result = result + CzzString::Format( "%s\n", urlEncode( url ).c_str() );
    }

    return result;
}

CzzString Cmp3M3UPlayList::urlEncode( const CzzString& url )
{
	CzzString result = url;
	int p = 0;

	while( (p = result.find( " " )) != -1 )
		result.replace( p, 1, "%20" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%85" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%84" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%96" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%A5" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%A4" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%B6" );

	while( (p = result.find( "'" )) != -1 )
		result.replace( p, 1, "%27" );

	while( (p = result.find( "&" )) != -1 )
		result.replace( p, 1, "%26" );

	while( (p = result.find( "," )) != -1 )
		result.replace( p, 1, "%2C" );

	while( (p = result.find( "!" )) != -1 )
		result.replace( p, 1, "%21" );

	while( (p = result.find( "(" )) != -1 )
		result.replace( p, 1, "%28" );

	while( (p = result.find( ")" )) != -1 )
		result.replace( p, 1, "%29" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%BC" );

	while( (p = result.find( "" )) != -1 )
		result.replace( p, 1, "%C3%9C" );

    while( (p = result.find( "[" )) != -1 )
		result.replace( p, 1, "%5B" );

    while( (p = result.find( "]" )) != -1 )
		result.replace( p, 1, "%5D" );

	return result; 
}

